local options = {
	type = "group",
	name = "SavedInstances",
	handler = SavedInstances,
	args = {
		General = {
			order = 1,
			type = "group",
			name = L["General settings"],
			desc = L["General settings"],
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["Track the instance IDs saved against your characters"],
				},
				MinimapIcon = {
					type = "toggle",
					name = L["Show minimap button"],
					desc = L["Show the SavedInstances minimap button"],
					order = 2,
					hidden = function() return not LDBIcon end,
					get = function(info) return not SavedInstances.db.MinimapIcon.hide end,
					set = function(info, value)
						SavedInstances.db.MinimapIcon.hide = not value
						if value then LDBIcon:Show("SavedInstances") else LDBIcon:Hide("SavedInstances") end
					end,
				},
				FreshToons = {
					type = "toggle",
					name = L["Show non-saved characters"],
					desc = L["Show characters that have not been saved to any instances"],
					order = 3,
					get = function(info)
							return SavedInstances.db.Tooltip.FreshToons
					end,
					set = function(info, value)
							SavedInstances.db.Tooltip.FreshToons = value
					end,
				},
				List = {
					order = 4,
					type = "multiselect",
					name = L["Always hide these characters"],
					get = function(info, toon)
						return SavedInstances.db.Toons[toon].Hide
					end,
					set = function(info, toon, value)
						SavedInstances.db.Toons[toon].Hide = value
					end,
					values = function()
						local list = { }
						local k, v
						for k, v in pairs(SavedInstances.db.Toons) do
							list[k] = k
						end
						return list
					end,
				},
				Indicators = {
					order = 5,
					type = "group",
					guiInline = true,
					name = L["Status indicators"],
					desc = L["Status indicators"],
					get = function(info)
							return SavedInstances.db.Indicators[info[#info]]
					end,
					set = function(info, value)
							SavedInstances.db.Indicators[info[#info]] = value
					end,
					args = {
						NormalIndicator = {
							order = 1,
							type = "select",
							name = L["Normal indicator"],
							values = Indicators,
						},
						NormalText = {
							order = 2,
							type = "input",
							name = L["Normal text"],
							multiline = false,
							disabled = function()
								return SavedInstances.db.Indicators.NormalIndicator ~= "TEXT"
							end,
						},
						NormalColor = {
							order = 3,
							type = "color",
							hasAlpha = false,
							name = L["Normal text color"],
							disabled = function()
								return SavedInstances.db.Indicators.NormalIndicator ~= "TEXT"
							end,
							get = function(info)
								local r = SavedInstances.db.Indicators[info[#info]][1]
								local g = SavedInstances.db.Indicators[info[#info]][2]
								local b = SavedInstances.db.Indicators[info[#info]][3]
								local a = SavedInstances.db.Indicators[info[#info]][4]
								return r, g, b, a
							end,
							set = function(info, r, g, b, a)
								SavedInstances.db.Indicators[info[#info]][1] = r
								SavedInstances.db.Indicators[info[#info]][2] = g
								SavedInstances.db.Indicators[info[#info]][3] = b
								SavedInstances.db.Indicators[info[#info]][4] = a
							end
						},
						HeroicIndicator = {
							order = 4,
							type = "select",
							name = L["Heroic indicator"],
							values = Indicators,
						},
						HeroicText = {
							order = 5,
							type = "input",
							name = L["Heroic text"],
							multiline = false,
							disabled = function()
								return SavedInstances.db.Indicators.HeroicIndicator ~= "TEXT"
							end,
						},
						HeroicColor = {
							order = 6,
							type = "color",
							hasAlpha = false,
							name = L["Heroic text color"],
							disabled = function()
								return SavedInstances.db.Indicators.HeroicIndicator ~= "TEXT"
							end,
							get = function(info)
								local r = SavedInstances.db.Indicators[info[#info]][1]
								local g = SavedInstances.db.Indicators[info[#info]][2]
								local b = SavedInstances.db.Indicators[info[#info]][3]
								local a = SavedInstances.db.Indicators[info[#info]][4]
								return r, g, b, a
							end,
							set = function(info, r, g, b, a)
								SavedInstances.db.Indicators[info[#info]][1] = r
								SavedInstances.db.Indicators[info[#info]][2] = g
								SavedInstances.db.Indicators[info[#info]][3] = b
								SavedInstances.db.Indicators[info[#info]][4] = a
							end
						},
					},
				},
			},
		},
		Instances = {
			order = 4,
			type = "group",
			name = L["Instance options"],
			desc = L["Instance options"],
			args = {
				OrderGroup = {
					order = 1,
					type = "group",
					guiInline = true,
					name = L["Instance order"],
					desc = L["Instance order"],
					get = function(info, instance)
						return SavedInstances.selectedInstance
					end,
					args = {
						Selected = {
							order = 1,
							type = "select",
							name = L["Instance to be moved"],
							set = function(info, instance)
								SavedInstances.selectedInstance = instance
							end,
							values = OrderedInstances,
						},
						Position = {
							order = 2,
							type = "select",
							name = L["Position in list"],
							set = function(info, instance)
								SetInstanceOrder(SavedInstances.selectedInstance, instance)
								SavedInstances.selectedInstance = nil
							end,
							values = OrderedInstances,
						},
					},
				},
				VisibilityGroup = {
					order = 2,
					type = "group",
					guiInline = true,
					name = L["Instance visibility"],
					desc = L["Instance visibility"],
					get = function(info)
							return SavedInstances.db.Tooltip[info[#info]]
					end,
					set = function(info, value)
							SavedInstances.db.Tooltip[info[#info]] = value
					end,
					args = {
						AllRaids = {
							type = "toggle",
							order = 1,
							name = L["Show all instances"],
							desc = L["Always show all instances, even if no characters are saved to them"],
						},
						NewInstanceShow = {
							type = "toggle",
							order = 2,
							name = L["Always show new instances"],
							desc = L["New instances are shown by default, even if no characters are saved to them"],
							disabled = function() return not SavedInstances.db.Tooltip.AllRaids end,
						},
						List = {
							type = "multiselect",
							order = 3,
							name = L["Show these instances when all instances are shown"],
							disabled = function() return not SavedInstances.db.Tooltip.AllRaids end,
							get = function(info, instance)
								local trimmedInstance = strsub(instance, 5)
								return SavedInstances.db.Instances[trimmedInstance].Show
							end,
							set = function(info, instance, value)
								local trimmedInstance = strsub(instance, 5)
								SavedInstances.db.Instances[trimmedInstance].Show = value
							end,
							values = function()
								local list = { }
								local i, v
								for i, v in ipairs(OrderedInstances()) do
									local prefix = format("%04d", i)
									list[prefix .. v] = v
								end
								return list
							end,
						},
					},
				},
			},
		},
	},
}
SavedInstances.Options = options